(function (tippy$1) {
  'use strict';

  tippy$1 = tippy$1 && tippy$1.hasOwnProperty('default') ? tippy$1['default'] : tippy$1;

  function _extends() {
    _extends = Object.assign || function (target) {
      for (var i = 1; i < arguments.length; i++) {
        var source = arguments[i];

        for (var key in source) {
          if (Object.prototype.hasOwnProperty.call(source, key)) {
            target[key] = source[key];
          }
        }
      }

      return target;
    };

    return _extends.apply(this, arguments);
  }

  /**
   * Safe .hasOwnProperty check, for prototype-less objects
   */

  function hasOwnProperty(obj, key) {
    return {}.hasOwnProperty.call(obj, key);
  }
  /**
   * Returns a value at a given index depending on if it's an array or number
   */

  function getValueAtIndexOrReturn(value, index, defaultValue) {
    if (Array.isArray(value)) {
      var v = value[index];
      return v == null ? Array.isArray(defaultValue) ? defaultValue[index] : defaultValue : v;
    }

    return value;
  }
  /**
   * Evaluates a function if one, or returns the value
   */

  function invokeWithArgsOrReturn(value, args) {
    return typeof value === 'function' ? value.apply(void 0, args) : value;
  }
  /**
   * Preserves the original function invocation when another function replaces it
   */

  function preserveInvocation(originalFn, currentFn, args) {
    if (originalFn && originalFn !== currentFn) {
      originalFn.apply(void 0, args);
    }
  }
  /**
   * Deletes properties from an object (pure)
   */

  function removeProperties(obj, keys) {
    var clone = _extends({}, obj);

    keys.forEach(function (key) {
      delete clone[key];
    });
    return clone;
  }
  /**
   * Creates an array from string of values separated by whitespace
   */

  function splitBySpaces(value) {
    return value.split(/\s+/).filter(Boolean);
  }

  var ARRAY_MISTAKE_ERROR = "\n  First argument to createSingleton() must be an *array* of tippy instances. The\n  passed value was a *single* tippy instance.\n";
  var EXISTING_SINGLETON_ERROR = "\n  The passed tippy instance(s) are already part of an existing singleton\n  instance. Make sure you destroy the previous singleton before calling\n  createSingleton() again.\n";
  var MISSING_TARGET_WARNING = "\n  You must specify a `target` prop indicating the CSS selector string\n  matching the target elements that should receive a tippy.\n";
  function createInvalidCreateSingletonArgumentError(arg) {
    return "\n    The first argument passed to createSingleton() must be an array of tippy\n    instances.\n\n    The passed value was: " + arg + "\n  ";
  }
  /**
   * Helpful wrapper around thrown errors
   */

  function throwErrorWhen(condition, message) {
    if (condition) {
      throw new Error(message);
    }
  }

  /**
   * Creates a delegate instance that controls the creation of tippy instances
   * for child elements (`target` CSS selector).
   */
  function delegate(targets, props) {
    {
      throwErrorWhen(!props || !props.target, MISSING_TARGET_WARNING);
    }

    var listeners = [];
    var childTippyInstances = [];
    var target = props.target;
    var nativeProps = removeProperties(props, ['target']);
    var trigger = props.trigger || tippy$1.defaultProps.trigger; // The user needs to specify their own enhanced tippy function to use extra
    // props
    // @ts-ignore

    var tippyConstructor = delegate.tippy || tippy$1;
    var returnValue = tippyConstructor(targets, _extends({}, nativeProps, {
      trigger: 'manual'
    }));

    function onTrigger(event) {
      if (event.target) {
        var targetNode = event.target.closest(target);

        if (targetNode) {
          var instance = tippyConstructor(targetNode, _extends({}, nativeProps, {
            showOnCreate: true
          }));

          if (instance) {
            childTippyInstances = childTippyInstances.concat(instance);
          }
        }
      }
    }

    function on(element, eventType, listener, options) {
      if (options === void 0) {
        options = false;
      }

      element.addEventListener(eventType, listener, options);
      listeners.push({
        element: element,
        eventType: eventType,
        listener: listener,
        options: options
      });
    }

    function addEventListeners(instance) {
      var reference = instance.reference;
      splitBySpaces(trigger).forEach(function (eventType) {
        switch (eventType) {
          case 'mouseenter':
            {
              on(reference, 'mouseover', onTrigger);
              break;
            }

          case 'focus':
            {
              on(reference, 'focusin', onTrigger);
              break;
            }

          case 'click':
            {
              on(reference, 'click', onTrigger);
            }
        }
      });
    }

    function removeEventListeners() {
      listeners.forEach(function (_ref) {
        var element = _ref.element,
            eventType = _ref.eventType,
            listener = _ref.listener,
            options = _ref.options;
        element.removeEventListener(eventType, listener, options);
      });
      listeners = [];
    }

    function applyMutations(instance) {
      var originalDestroy = instance.destroy;

      instance.destroy = function (shouldDestroyChildInstances) {
        if (shouldDestroyChildInstances === void 0) {
          shouldDestroyChildInstances = true;
        }

        if (shouldDestroyChildInstances) {
          childTippyInstances.forEach(function (instance) {
            instance.destroy();
          });
        }

        childTippyInstances = [];
        removeEventListeners();
        originalDestroy();
      };

      addEventListeners(instance);
      instance.setProps({
        trigger: 'manual'
      });
    }

    var instances = [].concat(returnValue);
    instances.forEach(applyMutations);
    return returnValue;
  }

  /**
   * Re-uses a single tippy element for many different tippy instances.
   * Replaces v4's `tippy.group()`.
   */
  function createSingleton(tippyInstances, optionalProps) {
    if (optionalProps === void 0) {
      optionalProps = {
        delay: 0
      };
    }

    {
      if (!Array.isArray(tippyInstances)) {
        throwErrorWhen(!tippyInstances, createInvalidCreateSingletonArgumentError(tippyInstances));
        throwErrorWhen( // @ts-ignore
        tippyInstances.reference && tippyInstances.reference._tippy, ARRAY_MISTAKE_ERROR);
      }

      var isAnyInstancePartOfExistingSingleton = tippyInstances.some(function (instance) {
        return hasOwnProperty(instance, '__singleton');
      });
      throwErrorWhen(isAnyInstancePartOfExistingSingleton, EXISTING_SINGLETON_ERROR);
      tippyInstances.forEach(function (instance) {
        Object.defineProperty(instance, '__singleton', {
          value: true,
          enumerable: false,
          configurable: true
        });
      });
    } // The user needs to specify their own enhanced tippy function to use extra
    // props
    // @ts-ignore


    var tippyConstructor = createSingleton.tippy || tippy$1;
    var singletonInstance = tippyConstructor(document.createElement('div'));
    var _optionalProps = optionalProps,
        delay = _optionalProps.delay;
    var showTimeout;
    var hideTimeout;

    function clearTimeouts() {
      clearTimeout(showTimeout);
      clearTimeout(hideTimeout);
    }

    tippyInstances.forEach(function (instance) {
      instance.__originalProps = {
        delay: instance.props.delay,
        onShow: instance.props.onShow,
        onTrigger: instance.props.onTrigger
      };
      instance.__originalClearDelayTimeouts = instance.clearDelayTimeouts;

      instance.clearDelayTimeouts = function () {
        instance.__originalClearDelayTimeouts();

        clearTimeouts();
      };

      instance.setProps({
        delay: delay,
        onShow: function onShow() {
          return false;
        },
        onTrigger: function onTrigger(_, event) {
          preserveInvocation(instance.__originalProps.onTrigger, instance.props.onTrigger, [instance, event]);
          singletonInstance.clearDelayTimeouts();
          singletonInstance.setProps(_extends({}, instance.props, {
            onShow: instance.__originalProps.onShow,
            triggerTarget: instance.reference
          }));
          var appendTo = instance.props.appendTo;
          singletonInstance.reference.referenceNode = appendTo === 'parent' ? instance.reference.parentNode : invokeWithArgsOrReturn(appendTo, [instance.reference]);

          singletonInstance.reference.getBoundingClientRect = function () {
            return instance.reference.getBoundingClientRect();
          };

          clearTimeouts(); // Edge case: if the tippy is currently hiding (but still mounted and
          // visible due to its opacity), it will slide to the new reference
          // element but fully to fade out before fading back in.
          // Also, we need to ensure the `popper` element doesn't set its
          // `transitionDuration` to 0ms, so it can transition smoothly

          if (!singletonInstance.state.isVisible && singletonInstance.state.isMounted) {
            singletonInstance.show(undefined, false);
          } else {
            showTimeout = setTimeout(function () {
              singletonInstance.show();
            }, getValueAtIndexOrReturn(delay, 0, tippy$1.defaultProps.delay));
          }
        }
      }); // Ensure the lifecycle functions can be updated

      instance.__originalSetProps = instance.setProps;

      instance.setProps = function (partialProps) {
        if (partialProps.onTrigger) {
          instance.__originalProps.onTrigger = partialProps.onTrigger;
        }

        if (partialProps.onShow) {
          instance.__originalProps.onShow = partialProps.onShow;
        }

        if (partialProps.delay !== undefined) {
          instance.__originalProps.delay = partialProps.delay;
        }

        instance.__originalSetProps(_extends({}, removeProperties(partialProps, ['onTrigger', 'onShow', 'delay']), {
          delay: delay
        }));
      };
    });
    var originalSetProps = singletonInstance.setProps;

    singletonInstance.setProps = function (partialProps) {
      if (partialProps.delay !== undefined) {
        delay = partialProps.delay;
        tippyInstances.forEach(function (instance) {
          instance.setProps({
            delay: delay
          });
        });
      }

      originalSetProps(partialProps);
    };

    var originalDestroy = singletonInstance.destroy;

    singletonInstance.destroy = function ( // ESLint parser and TypeScript are condradicting each other here for some
    // reason
    // eslint-disable-next-line
    shouldDestroyPassedInstances) {
      if (shouldDestroyPassedInstances === void 0) {
        shouldDestroyPassedInstances = true;
      }

      tippyInstances.forEach(function (instance) {
        if (instance.state.isDestroyed) {
          return;
        } // Restore the instances to their original state


        instance.clearDelayTimeouts = instance.__originalClearDelayTimeouts;
        instance.setProps = instance.__originalSetProps;
        instance.setProps(instance.__originalProps);
        delete instance.__originalClearDelayTimeouts;
        delete instance.__originalProps;
        delete instance.__originalSetProps;

        {
          delete instance.__singleton;
        }

        if (shouldDestroyPassedInstances) {
          instance.destroy();
        }
      });
      originalDestroy();
    };

    return singletonInstance;
  }

  /* eslint-disable no-undef */
  tippy.delegate = delegate;
  tippy.createSingleton = createSingleton;

}(tippy));
//# sourceMappingURL=tippy-addons.iife.js.map
